@@ -1,10 +1,14 @@
 import asyncio
+import time
 from abc import ABC, abstractmethod
 from statistics import mean
 
+import nest_asyncio
 import numpy as np
 import requests
+from aiohttp import ClientSession
 from pydantic import BaseModel
+from requests.exceptions import RequestException
 from sentence_transformers import SentenceTransformer, util
 
 from llmstudio.engine.config import EngineConfig, RouteType
@@ -38,6 +42,7 @@ class LLMModel(ABC):
         api_key: str = None,
         api_secret: str = None,
         api_region: str = None,
+        tests: dict = {},
         engine_config: EngineConfig = EngineConfig(),
     ):
         """
@@ -47,12 +52,14 @@ class LLMModel(ABC):
             model_name (str): The name of the model to be used.
             api_key (str, optional): The API key for authentication. Default is None.
             api_secret (str, optional): The API secret for enhanced security. Default is None.
+            tests (dict, optional): Dictionary of batch of tests to be used when running tests or evaluation. Default is Empty Dict.
             api_region (str, optional): The API region for interfacing. Default is None.
         """
         self.model_name = model_name
         self.api_key = api_key
         self.api_secret = api_secret
         self.api_region = api_region
+        self.tests = tests
         self.validation_url = f"{str(engine_config.routes_endpoint)}/{RouteType.LLM_VALIDATION.value}/{self.PROVIDER}"
         self.chat_url = (
             f"{str(engine_config.routes_endpoint)}/{RouteType.LLM_CHAT.value}/{self.PROVIDER}"
@@ -64,18 +71,29 @@ class LLMModel(ABC):
             "Please provide api_key parameter or set the specific environment variable."
         )
 
-    def _check_api_access(self):
-        response = requests.post(
-            self.validation_url,
-            json={
-                "model_name": self.model_name,
-                "api_key": self.api_key,
-            },
-            headers={"Content-Type": "application/json"},
-            timeout=10,
+    def _check_api_access(self, max_retries=3, delay=0.5):
+        retries = 0
+        while retries < max_retries:
+            try:
+                response = requests.post(
+                    self.validation_url,
+                    json={
+                        "model_name": self.model_name,
+                        "api_key": self.api_key,
+                    },
+                    headers={"Content-Type": "application/json"},
+                    timeout=10,
+                )
+                if not response.json():
+                    raise ValueError(f"The API key doesn't have access to {self.model_name}")
+                return  # Successful API check
+            except RequestException:
+                retries += 1
+                time.sleep(delay)  # Wait before retrying
+
+        raise ValueError(
+            "Max retries reached. The API key doesn't have access to {self.model_name}"
         )
-        if not response.json():
-            raise ValueError(f"The API key doesn't have access to {self.model_name}")
 
     @abstractmethod
     def validate_parameters(self, parameters: BaseModel) -> BaseModel:
@@ -89,7 +107,7 @@ class LLMModel(ABC):
             BaseModel: Validated/adjusted parameters encapsulated in a Pydantic model.
         """
 
-    def chat(self, chat_input: str, parameters: BaseModel = None, is_stream: bool = False):
+    def chat(self, chat_input: str, parameters: BaseModel = None, is_stream: bool = False,safety_margin = None, custom_max_tokens = None):
         """
         Initiate a chat interaction with the language model.
 
@@ -124,13 +142,121 @@ class LLMModel(ABC):
                 "chat_input": chat_input,
                 "parameters": validated_params,
                 "is_stream": is_stream,
+                "safety_margin": safety_margin,
+                "custom_max_token": custom_max_tokens
             },
             headers={"Content-Type": "application/json"},
-            timeout=10,
+            timeout=30,
         )
 
         return response.json()
 
+    # Async (wannabe parallel) run tests
+    async def chat_async(
+        self, chat_input: str, parameters: BaseModel = None, is_stream: bool = False
+    ):
+        async with ClientSession() as session:
+            async with session.post(
+                self.chat_url,
+                json={
+                    "model_name": self.model_name,
+                    "api_key": self.api_key,
+                    "api_secret": self.api_secret,
+                    "api_region": self.api_region,
+                    "chat_input": chat_input,
+                    "parameters": parameters,
+                    "is_stream": is_stream,
+                },
+            ) as response:
+                return await response.json()
+
+    async def run_tests_async(
+        self, tests: dict = {}, parameters: dict = {}, is_stream: bool = False
+    ):
+        if not tests:
+            tests = self.tests
+        tests = self.validate_tests(tests)
+
+        async def run_single_test(key):
+            test, gt_answer = tests[key].values()
+            print(f"Running test... Key: {key} / Test: {test} / GT Answer: {gt_answer}")
+            answer = await self.chat_async(test, parameters=parameters, is_stream=is_stream)
+            print(f"Answer: {answer}")
+            return key, answer
+
+        tasks = [run_single_test(key) for key in tests]
+        test_responses = await asyncio.gather(*tasks)
+        return {k: v for k, v in test_responses}
+
+    def run_tests(self, tests: dict = {}, parameters: dict = {}, is_stream: bool = False):
+        loop = asyncio.get_event_loop()
+        if loop.is_running():
+            # Jupyter Notebook; use a workaround since loop is already running
+            nest_asyncio.apply()
+            return loop.run_until_complete(self.run_tests_async(tests, parameters, is_stream))
+        else:
+            # Standalone script; standard approach
+            return asyncio.run(self.run_tests_async(tests, parameters, is_stream))
+
+    def set_tests(self, tests: dict = {}):
+        self.tests = tests
+
+    def validate_tests(self, tests):
+        """
+        Validate that the tests are not empty
+
+        Args:
+            tests: The tests to validate.
+
+        Returns:
+            str: The validated tests.
+
+        Raises:
+            ValueError: If the tests are empty.
+        """
+        if not tests:
+            raise ValueError(f"tests should not be empty")
+
+        correct_format = (
+            """'test_1': {'question': 'What is the capital of Portugal', 'answer': 'Lisbon'}"""
+        )
+
+        if not isinstance(tests, dict):
+            raise ValueError(f"tests is not a dict")
+        for key, value in tests.items():
+            if not isinstance(key, str):
+                raise ValueError(
+                    f"key of tests is not a string. Example of correct format: {correct_format}"
+                )
+            if not isinstance(value, dict):
+                raise ValueError(
+                    f"value of each test is not a Dictionary. Example of correct format: {correct_format}"
+                )
+            if "question" not in value or "answer" not in value:
+                raise ValueError(f"tests value should be in format: {correct_format}")
+            if not (isinstance(value["question"], str) and isinstance(value["answer"], str)):
+                raise ValueError(f"question and answer should be strings")
+        return tests
+
+    # Sequential code for running tests
+    """
+    def run_tests(self, tests: dict = {}, parameters: dict = {}, is_stream: bool = False):
+
+        if not tests:
+            tests = self.tests
+
+        tests = self.validate_tests(tests)
+
+        test_responses = {}
+        for key in tests:
+            test, gt_answer = tests[key].values()
+            print(f"Key: {key} / Test: {test} / GT Answer: {gt_answer}")
+            response = self.chat(test,parameters=parameters,is_stream=is_stream)
+            test_responses[key] = response
+
+        return test_responses
+    """
+
 
 class LLMClient(ABC):
     """
